@@ -1,6 +1,6 @@
 use std::future::Future;
 
-use crate::{Endpoint, IntoResponse, Request, Response, Result};
+use crate::{Endpoint, IntoResponse, Request, Result};
 
 /// Endpoint for the [`and_then`](super::EndpointExt::and_then) method.
 pub struct AndThen<E, F> {
@@ -16,19 +16,19 @@ impl<E, F> AndThen<E, F> {
 }
 
 #[async_trait::async_trait]
-impl<E, F, Fut, R> Endpoint for AndThen<E, F>
+impl<E, F, Fut, R, R2> Endpoint for AndThen<E, F>
 where
-    E: Endpoint,
-    F: Fn(Response) -> Fut + Send + Sync + 'static,
-    Fut: Future<Output = Result<R>> + Send + 'static,
+    E: Endpoint<Output = Result<R>>,
+    F: Fn(R) -> Fut + Send + Sync + 'static,
+    Fut: Future<Output = Result<R2>> + Send + 'static,
     R: IntoResponse,
+    R2: IntoResponse,
 {
-    async fn call(&self, req: Request) -> Result<Response> {
+    type Output = Result<R2>;
+
+    async fn call(&self, req: Request) -> Self::Output {
         match self.inner.call(req).await {
-            Ok(resp) => (self.f)(resp)
-                .await
-                .map_err(Into::into)
-                .and_then(IntoResponse::into_response),
+            Ok(resp) => (self.f)(resp).await,
             Err(err) => Err(err),
         }
     }
